clear all

%This script must not be considered as a complete program for 
%calculation of change in attitude and position, but as result of
%analysis of navigation equations.

%To achieve precise Scale factors, Offsets and misalingment matrices
%calibration data is loaded and values calculated before navigation
%loop starts.

%-Gyro X---------------------------------------------------
load ccw_x_90_nr1
g_x_ccw = mean(g_x);
load cw_x_90_nr1
g_x_cw = mean(g_x);
rate=90*(pi/180); %rotation rate
O_gx = (g_x_ccw + g_x_cw)/2;  %Offset 
Sf_gx = (g_x_ccw - g_x_cw)/(2*rate); %Scale factor
%--------------------------------------------------------------

%-Gyro Y---------------------------------------------------
load ccw_y_90_nr1
g_y_ccw = mean(g_y);
load cw_y_90_nr1
g_y_cw = mean(g_y);
O_gy = (g_y_ccw + g_y_cw)/2; %Offset
Sf_gy = (g_y_ccw - g_y_cw)/(2*rate)*(-1); %Scale factor, *-1 because of NED
%--------------------------------------------------------------

%-Gyro Z---------------------------------------------------
load ccw_z_90_nr1
g_z_ccw = mean(g_z);
load cw_z_90_nr1
g_z_cw = mean(g_z);
O_gz = (g_z_ccw + g_z_cw)/2; %Offset
Sf_gz = (g_z_ccw - g_z_cw)/(2*rate)*(-1); %Scale factor, *-1 because of NED
%---------------------------------------------------------------

%-Gyro-----------------------------------------------------
O_g = [ O_gx ; O_gy ; O_gz ]; %Offset vector
Sf_g = [ Sf_gx ; Sf_gy ; Sf_gz ]; %Scale factor vector
%----------------------------------------------------------

%-Misalingment matrix gyro---------------------------------
load ccw_x_90_nr1
SM_g(:,1) = [ (mean(g_x) - O_gx) / Sf_gx / (pi/2) ;
             (mean(g_y) - O_gy) / Sf_gy / (pi/2) ;
             (mean(g_z) - O_gz) / Sf_gz / (pi/2) ];

load ccw_y_90_nr1
SM_g(:,2) = [ (mean(g_x) - O_gx) / Sf_gx / (pi/2) ;
             (mean(g_y) - O_gy) / Sf_gy / (pi/2) ;
             (mean(g_z) - O_gz) / Sf_gz / (pi/2) ];
         
load ccw_z_90_nr1
SM_g(:,3) = [ (mean(g_x) - O_gx) / Sf_gx / (pi/2) ;
             (mean(g_y) - O_gy) / Sf_gy / (pi/2) ;
             (mean(g_z) - O_gz) / Sf_gz / (pi/2) ];
%------------------------------------------------------------

%--Accelerometer axis X-----------------------------------
load a_x_pos_n01
pos_x=mean(a_x);
load a_x_neg_n01
neg_x=mean(a_x);
g = 9.8218; %gravitation
O_ax = (pos_x+(neg_x))/2; %Offset
Sf_ax = (pos_x + (-1*neg_x))/(2*g); %Scale factor
%---------------------------------------------------------

%--Accelerometer axis Y---------------------------------
load a_y_pos_n01
pos_y=mean(a_y);
load a_y_neg_n01
neg_y=mean(a_y);
O_ay = (pos_y+(neg_y))/2;  %Offset
Sf_ay = (pos_y + (-1*neg_y))/(2*g)*(-1); %Scale factor, *-1 because of NED
%---------------------------------------------------------

%--Accelerometer axis Z---------------------------------
load a_z_pos_n01
pos_z=mean(a_z);
load a_z_neg_n01
neg_z=mean(a_z);
O_az = (pos_z+(neg_z))/2; %Offset
Sf_az = (pos_z + (-1*neg_z))/(2*g)*(-1); %Scale factor, *-1 because of NED
%--------------------------------------------------------------------------

%--Accelerometer------------------------------------
O_a = [O_ax O_ay O_az]; %Offser vector
Sf_a = [Sf_ax Sf_ay Sf_az]; %Scale factor vector
%----------------------------------------------------

%-Misalingment matrix accelerometer---------------------------------
load a_x_pos_n01
SM_a(:,1) = [ ((mean(a_x) - O_ax) / Sf_ax) / g ;
             ((mean(a_y) - O_ay) / Sf_ay) / g ; 
             ((mean(a_z) - O_az) / Sf_az) / g ]; 

load a_y_pos_n01
SM_a(:,2) = [ ((mean(a_x) - O_ax) / Sf_ax) / g ; 
             ((mean(a_y) - O_ay) / Sf_ay) / g ; 
             ((mean(a_z) - O_az) / Sf_az) / g ];
         
load a_z_pos_n01
SM_a(:,3) = [ ((mean(a_x) - O_ax) / Sf_ax) / g ; 
             ((mean(a_y) - O_ay) / Sf_ay) / g ; 
             ((mean(a_z) - O_az) / Sf_az) / g ];
%------------------------------------------------------------------

%--Offset manipulation---------------------------------
% load kal23mai %filename calibration data
% O_ax = mean(a_x);
% O_ay = mean(a_y);
% O_az = mean(a_z);
% O_gx = mean(g_x);
% O_gy = mean(g_y);
% O_gz = mean(g_z);
%-----------------------------------------------------------

%---Initial Euler angle calculation-------------------------------------------------
load 21mai %filename attitude data
acc_skew_x=((mean(a_x)-O_ax)/Sf_ax); % along x-axis in body coordinates 
acc_skew_y=((mean(a_y)-O_ay)/Sf_ay); % along y-axis in body coordinates 
acc_skew_z=((mean(a_z)-O_az)/Sf_az); % along z-axis in body coordinates 
    
acc_skew=[acc_skew_x; acc_skew_y; acc_skew_z];
acc_unskew=inv(SM_a)*acc_skew;%
    
ax=acc_unskew(1);
ay=acc_unskew(2);
az=acc_unskew(3);
    
aks_for_rot=[ax; ay; az]

g= sqrt((ax^2)+(ay^2)+(az^2));

phi=atan(ay/az);
theta=-asin(ax/g)+pi;
psi=0;

Euler_angles=[phi*(180/pi); theta*(180/pi); psi*(180/pi)];
Euler_check=[-g*sin(theta); g*cos(theta)*sin(phi); g*cos(theta)*cos(phi)];
%--------------------------------------------------------------------------


%-Initial data-----------
%load filename %filename sensor navigation data
g = 9.8218;
Fs=100;
dt=1/Fs;
shift=1;
pos_l=[0;0;0]; %Quarternion
posi_l=[0;0;0]; %DCM
vel_b=[0;0;0]; %initial velocity body ramme

R_b2t=[cos(psi)*cos(theta) -sin(psi)*cos(phi)+cos(psi)*sin(theta)*sin(phi) sin(psi)*sin(phi)+cos(psi)*sin(theta)*cos(phi);
     sin(psi)*cos(theta) cos(psi)*cos(phi)+sin(psi)*sin(theta)*sin(phi) -cos(psi)*sin(phi)+sin(psi)*sin(theta)*cos(phi);
     -sin(theta) cos(theta)*sin(phi) cos(theta)*cos(phi)];
   
a=cos(phi/2)*cos(theta/2)*cos(psi/2)+sin(phi/2)*sin(theta/2)*sin(psi/2);
b=sin(phi/2)*cos(theta/2)*cos(psi/2)-cos(phi/2)*sin(theta/2)*sin(psi/2);
c=cos(phi/2)*sin(theta/2)*cos(psi/2)+sin(phi/2)*cos(theta/2)*sin(psi/2);
d=cos(phi/2)*cos(theta/2)*sin(psi/2)-sin(phi/2)*sin(theta/2)*cos(psi/2);
     
R_b2t_q=[a^2+b^2-c^2-d^2 2*(b*c-a*d) 2*(b*d+a*c);
        2*(b*c+a*d) a^2-b^2+c^2-d^2 2*(c*d-a*b);
        2*(b*d-a*c) 2*(c*d+a*b) a^2-b^2-c^2+d^2];
    
%acc_after_rotDCM=R_b2t*aks_for_rot;
%acc_after_rotquar=R_b2t_q*aks_for_rot;
    
%---Euler angles from Quarternions -------------------        
 phi_2=atan(R_b2t_q(3,2)/R_b2t_q(3,3))*(180/pi);
 theta_2=asin(-R_b2t_q(3,1))*(180/pi);
 psi_2=atan(R_b2t_q(2,1)/R_b2t_q(1,1))*(180/pi);
 Euler_start=[phi_2; theta_2; psi_2]
% 
% phi_3=atan((2*(a*b+c*d))/(a^2-b^2-c^2+d^2))
% theta_3=asin((2*(a*c-d*b)))
% psi_3=atan((2*(a*d+b*c))/(a^2+b^2-c^2-d^2))
%------------------------------------------------------

e=a^2+b^2+c^2+d^2;
vel_l = R_b2t_q*vel_b;%Quarternion
hast_l = R_b2t*vel_b;%DCM

%--Filter function--------------------------------
%      f_a=1;% FIR
%      Wn=30/Fs;
%      f_b = fir1(30,Wn*2); %hamming window default
%      g_xf = filtfilt(f_b,f_a,g_x);
%      g_yf = filtfilt(f_b,f_a,g_y);
%      g_zf = filtfilt(f_b,f_a,g_z);
%---------------------------------------------------
acc_l(:,1)=[0; 0; 0]; acc_l(:,2)=[0; 0; 0]; acc_l(:,3)=[0; 0; 0]; 
vel_l(:,1)=[0; 0; 0]; vel_l(:,2)=[0; 0; 0]; vel_l(:,3)=[0; 0; 0];
aks_l(:,1)=[0; 0; 0]; aks_l(:,2)=[0; 0; 0]; aks_l(:,3)=[0; 0; 0]; 
hast_l(:,1)=[0; 0; 0]; hast_l(:,2)=[0; 0; 0]; hast_l(:,3)=[0; 0; 0];

%---Navigation loop-------------------------------------------------------

for i =4:3003
    %--Accelerometer--
    acc_skew_x=((a_x(i-3)-O_ax)/Sf_ax); % along x-axis in body coordinates
    acc_skew_y=((a_y(i-3)-O_ay)/Sf_ay); % along y-axis in body coordinates
    acc_skew_z=((a_z(i-3)-O_az)/Sf_az); % along z-axis in body coordinates
    
    acc_skew=[acc_skew_x; acc_skew_y; acc_skew_z];

    acc_unskew=inv(SM_a)*acc_skew;
    ax=acc_unskew(1);
    ay=acc_unskew(2);
    az=acc_unskew(3);
    
    %--Gyro--
%     p_m=(g_xf(i-3)+g_xf(i-2)+g_xf(i-1)+g_xf(i)+g_xf(i+1)+g_xf(i+2)+g_xf(i+3))/7;
%     q_m=(g_yf(i-3)+g_yf(i-2)+g_yf(i-1)+g_yf(i)+g_yf(i+1)+g_yf(i+2)+g_yf(i+3))/7;
%     r_m=(g_zf(i-3)+g_zf(i-2)+g_zf(i-1)+g_zf(i)+g_zf(i+1)+g_zf(i+2)+g_zf(i+3))/7;

    p_m=(g_x(i-3));
    q_m=(g_y(i-3));
    r_m=(g_z(i-3));
    
    p_s=(p_m-O_gx)/Sf_gx;     %roll
    q_s=(q_m-O_gy)/Sf_gy;     %pitch
    r_s=(r_m-O_gz)/Sf_gz;     %heading
    
    gyro_skew=[p_s; q_s; r_s];

    gyro_unskew=inv(SM_g)*gyro_skew;

    p=gyro_unskew(1);
    q=gyro_unskew(2);
    r=gyro_unskew(3);
    
%---Direction Cosine matrix update-----------------------------------------
%      R_b2t=(R_b2t*[0 -r*dt q*dt;
%                  r*dt 0 -p*dt;
%                  -q*dt p*dt 0])+R_b2t;
%--------------------------------------------------------------------------
    
%--Taylor update DCM-------------------------------------------------------
    sigma=sqrt(p^2+q^2+r^2);
    if sigma==0
        R_b2t=R_b2t; 
    else
        A=[0 -r*dt q*dt;
           r*dt 0 -p*dt;
          -q*dt p*dt 0];
  
        dt2=0.0001;
      
        A2=[-(q^2+r^2)*dt2 p*q*dt2 p*r*dt2;
            p*q*dt2 -(p^2+r^2)*dt2 q*r*dt2;
            p*r*dt2 q*r*dt2 -(q^2+r^2)*dt2];
    
        I=[1 0 0; 0 1 0; 0 0 1];
        R_b2t=R_b2t*(I+(sin(sigma)/sigma)*A+((1-cos(sigma))/sigma^2)*A2);
    end 
    
%--Quaternion update-------------------------------------------------------
    abcd = 0.5*[0  -p*dt  -q*dt  -r*dt;  
                 p*dt  0   r*dt  -q*dt;   
                  q*dt  -r*dt  0  p*dt;  
                 r*dt  q*dt  -p*dt  0] * [a;b;c;d] + [a;b;c;d]; 
    
    a=abcd(1); b=abcd(2); c=abcd(3); d=abcd(4);

%     a=(-0.5*(b*p+c*q+d*r)*dt)+a;
%     b=(0.5*(a*p-d*q+c*r)*dt)+b;
%     c=(0.5*(d*p+a*q-b*r)*dt)+c;
%     d=(-0.5*(c*p-b*q-a*r)*dt)+d;
              
    R_b2t_q=[a^2+b^2-c^2-d^2 2*(b*c-a*d) 2*(b*d+a*c);
             2*(b*c+a*d) a^2-b^2+c^2-d^2 2*(c*d-a*b);
            2*(b*d-a*c) 2*(c*d+a*b) a^2-b^2-c^2+d^2];       

%--Normalization ----------------------------------------------------------
    abcd=[a;b;c;d];
    e_for_normali=sqrt(a^2+b^2+c^2+d^2);
    abcd = abcd/(sqrt(a^2+b^2+c^2+d^2));
    a=abcd(1); b=abcd(2); c=abcd(3); d=abcd(4);
    e_etter_normal=sqrt(a^2+b^2+c^2+d^2);
%--------------------------------------------------------------------------
    
    force_b = [ax; ay; az];
    
    force_t = R_b2t_q*force_b; %Quarternion
    kraft_t = R_b2t*force_b; % DCM

    acc_l(:,i)=force_t+[0;0;g]; % gravity compensation Quarternion
    aks_l(:,i)=kraft_t+[0;0;g]; % gravity compensation DCM
   
    vel_l(:,i)=((acc_l(:,i-3)+2*acc_l(:,i-2)+2*acc_l(:,i-1)+acc_l(:,i))/6)*dt+vel_l(:,i-1);%Runge Kutta
    %vel_l=(acc_l*dt)+vel_l;   %Quarternion
    
    hast_l(:,i)=((aks_l(:,i-3)+2*aks_l(:,i-2)+2*aks_l(:,i-1)+aks_l(:,i))/6)*dt+hast_l(:,i-1);%Runge Kutta
    %hast_l=(aks_l*dt)+hast_l; %DCM
    
    pos_l=((vel_l(:,i-3)+2*vel_l(:,i-2)+2*vel_l(:,i-1)+vel_l(:,i))/6)*dt+pos_l;%Runge Kutta
    %pos_l=(vel_l*dt)+pos_l;   %Quarternion
    
    posi_l=((hast_l(:,i-3)+2*hast_l(:,i-2)+2*hast_l(:,i-1)+hast_l(:,i))/6)*dt+posi_l;%Runge Kutta
    %posi_l=(hast_l*dt)+posi_l;%DCM
    
%--store position vector----------- 
    %--pos vector quarternion--
    pos_N(i)=pos_l(1);
    pos_E(i)=pos_l(2);
    pos_D(i)=pos_l(3);
 
    %--pos vector DCM--
    posi_N(i)=posi_l(1);
    posi_E(i)=posi_l(2);
    posi_D(i)=posi_l(3);
%------------------------------------

%--singularity free Euler angle update-------------------------------------
    theta_2=asin((2*(a*c-d*b)))*(180/pi);
   
    if (theta_2 > 89) && (theta_2 < 91) %theta near +pi/2
       
        if shift==1
            psi_2=(atan((R_b2t_q(2,3)-R_b2t_q(1,2))/(R_b2t_q(1,3)+R_b2t_q(2,2))))*(180/pi)+phi_2;
            shift=shift*(-1);
            i;
        else
            phi_2=psi_2-(atan((R_b2t_q(2,3)-R_b2t_q(1,2))/(R_b2t_q(1,3)+R_b2t_q(2,2))))*(180/pi);
            shift=shift*(-1);
            i2=i;
        end
           
    elseif (theta_2 > -91) && (theta_2 < -89) %theta near -pi/2
     
        if shift==1
            psi_2=atan((R_b2t_q(2,3)+R_b2t_q(1,2))/(R_b2t_q(1,3)-R_b2t_q(2,2)))*(180/pi)-phi_2;
            shift=shift*(-1);
        else
            phi_2=atan((R_b2t_q(2,3)+R_b2t_q(1,2))/(R_b2t_q(1,3)-R_b2t_q(2,2)))*(180/pi)-psi_2;
            shift=shift*(-1);
        end

    else
       
       phi_2=atan((2*(a*b+c*d))/(a^2-b^2-c^2+d^2))*(180/pi);
       psi_2=atan((2*(a*d+b*c))/(a^2+b^2-c^2-d^2))*(180/pi);
       
    end
%--------------------------------------------------------------------------

%--Drift analysis----------------------------------------------------------
    drift1(i)=kraft_t(1);
    drift2(i)=kraft_t(2);
    drift3(i)=kraft_t(3);
    drift4(i)=phi_2;
    drift5(i)=theta_2;
    drift6(i)=psi_2;
%--------------------------------------------------------------------------
    
end

%-Values after for-loop
e_etter=sqrt(a^2+b^2+c^2+d^2);
DCM_etter=R_b2t;
Quarternion_etter=R_b2t_q;
%-----------------------------

%---Euler angles from quarternion-------------------        
 phi_2;%=atan(R_b2t_q(3,2)/R_b2t_q(3,3))*(180/pi);
 theta_2;%=asin(-R_b2t_q(3,1))*(180/pi);
 psi_2;%=atan(R_b2t_q(2,1)/R_b2t_q(1,1))*(180/pi);
 Euler_end=[phi_2; theta_2; psi_2];
 
%--Euler angles from DCM
phi_dcm=atan(R_b2t(3,2)/R_b2t(3,3))*(180/pi);
theta_dcm=asin(-R_b2t(3,1))*(180/pi);
psi_dcm=atan(R_b2t(2,1)/R_b2t(1,1))*(180/pi);

figure (1)
clf 
plot3(posi_N,posi_E,posi_D,'-o')
xlabel('nord meter')
ylabel('st meter')
zlabel('ned meter')
title('RCM metode')
%  xlim([0 390])
%  ylim([0 390])
%  zlim([0 390])
grid on
axis square

figure (2)
clf 
plot3(pos_N,pos_E,pos_D,'-o')
xlabel('nord meter')
ylabel('st meter')
zlabel('ned meter')
title('Kvarternion metode')
%xlim([6 -4])
%ylim([400 0])
%zlim([-50 150])
grid on
axis square

timevec=0:dt:((i-3)*dt)-dt;

 figure(3)
 subplot(3,1,1);plot(timevec,a_x(1:i-3))
 title('rdata acc_x')
 xlabel('time [sec]')
 ylabel('acc amplitude')
 subplot(3,1,2);plot(timevec,a_y(1:i-3))
 title('rdata rdata acc_y')
 xlabel('time [sec]')
 ylabel('acc amplitude')
 subplot(3,1,3);plot(timevec,a_z(1:i-3))
 title('rdata  acc_z')
 xlabel('time [sec]')
 ylabel('acc amplitude')
 
 figure(4)
 subplot(3,1,1);plot(timevec,g_x(1:i-3))
 title('rdata gyro x')
 xlabel('time [sec]')
 ylabel('gyro amplitude')
 subplot(3,1,2);plot(timevec,g_y(1:i-3))
 title('rdata gyro y')
 xlabel('time [sec]')
 ylabel('gyro amplitude')
 subplot(3,1,3);plot(timevec,g_z(1:i-3))
 title('rdata gyro z')
 xlabel('time [sec]')
 ylabel('gyro amplitude')

figure(5)
subplot(3,1,1);plot(timevec,drift1(4:i))
 title('Kompenserte aks data')
 xlabel('tid [sek]')
 ylabel('Grader per sekund')
 subplot(3,1,2);plot(timevec,drift2(4:i))
 title('Kompenserte aks data')
 xlabel('tid [sek]')
 ylabel('Grader per sekund')
 subplot(3,1,3);plot(timevec,drift3(4:i))
 title('Kompenserte aks data')
 xlabel('tid [sek]')
 ylabel('Grader per sekund')
 
 figure(6)
 subplot(3,1,1);plot(timevec,drift4(4:i))
 title('Eulervinkel phi')
 xlabel('tid [sek]')
 ylabel('vinkel [grader]')
 subplot(3,1,2);plot(timevec,drift5(4:i))
 title('Eulervinkel theta')
 xlabel('tid [sek]')
 ylabel('vinkel [grader]')
 subplot(3,1,3);plot(timevec,drift6(4:i))
 title('Eulervinkel psi')
 xlabel('tid [sek]')
 ylabel('vinkel [grader]')
