/************************************************************************** 
Name: 		flash.c
Comments:	Functions for initializing, writing to and reading 
		the M25P80 FLASH-memory from ST.
**************************************************************************/ 

#include <mega128.h>
#include <delay.h>
#include <spi.h>
#include "flash.h"
#include "isr.h"


//Check the statusregister and wait until flash is ready
void wait_flash_rdy(void)
{
unsigned char status = 0;
unsigned char klar = 0x01;

ACTIVATE_FLASH;
spi(RDSR);
while (klar == 0x01)
	{
	status = spi(0x00);
	klar = (status & 0x01);
	}
DEACTIVATE_FLASH;
}


//Write enable
void write_enable(void)
{
ACTIVATE_FLASH;
spi(WREN);
DEACTIVATE_FLASH;
}


//Write disable   
//Not in use
void write_disable(void)
{
ACTIVATE_FLASH;
spi(WRDI);
DEACTIVATE_FLASH;
}


//Pageprogram in flash 
//Programs one byte at a time.
void PageProgram(char data, unsigned long int adress)
{
wait_flash_rdy();

write_enable();

ACTIVATE_FLASH;
spi(PP);
spi(adress>>16);
spi(adress>>8);
spi(adress);
spi(data);	
DEACTIVATE_FLASH;
}
 

//Delete the flash 
//This takes a long time..    10+ sec
void bulk_erase(void)
{
wait_flash_rdy();

write_enable();

//BULK ERASE
ACTIVATE_FLASH;
spi(BE);
DEACTIVATE_FLASH;
wait_flash_rdy();
}


//Read flash page by page (not in use).
void flash_page_to_UART(int page_nr)
{
unsigned char i, data;
unsigned long int start_adress;

start_adress = (page_nr*256);

wait_flash_rdy();

ACTIVATE_FLASH;
spi(READ);
spi(start_adress>>16);
spi(start_adress>>8);
spi(start_adress);
for(i=0; i<255; i++)
	{
	data=spi(0x00);
	while ((UCSR1A & DATA_REGISTER_EMPTY)==0);
        UDR1=data;
	}	
DEACTIVATE_FLASH;
}