/**************************************************************************
Project : PicoTracker (Prosjektoppgave)
Version : 3
Date    : 14.11.2005
Author  : Tore Johan Hilund and Tor Ivar Hystad         
Company : NTNU and FFI  
Compiler: CodeVisionAVR V1.23.7 Standard

Chip type           : ATmega128
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
Internal SRAM size  : 4096
External SRAM size  : 0
Data Stack size     : 1024
**************************************************************************/

#include <mega128.h>
#include <delay.h>
#include <spi.h>
#include <string.h>
#include <stdlib.h>
#include "flash.h"
#include "display.h"
#include "nmea.h"
#include "time.h"

#define RXB8 1
#define TXB8 0
#define UPE 2
#define OVR 3
#define FE 4
//#define UDRE 5
#define RXC 7

#define FRAMING_ERROR (1<<FE)
#define PARITY_ERROR (1<<UPE)
#define DATA_OVERRUN (1<<OVR)
//#define DATA_REGISTER_EMPTY (1<<UDRE)
#define RX_COMPLETE (1<<RXC)

#define END_NMEA 0xA
#define CR 0xD
#define START_NMEA '$'
#define flash_size 1048500 //really 1048576 bytes, but use 1048500 for safety reasons

//battery defines
#define charge_disable PORTB.7=1  
#define charge_enable PORTB.7=0
#define charge_current_high PORTG.1=1
#define charge_current_low PORTG.1=0
#define TTE_enable PORTB.6=0
#define TTE_disable PORTB.6=1
#define STAT1 PORTB.4
#define STAT2 PORTB.5

//Functions in main.c
char CHAR2BCD2(char input);
void charger(void);
void check_batt(void);

//BRUKES IKKE 
//void find_start_adresse(void); 

// Global variables
unsigned char index1, nmeaID;

//Variable til/fra ISR
volatile char nmea_data[82];
volatile char index;
volatile char temp_data[82];
volatile char i;
volatile char tracking;
volatile char flash_delete = 0;
volatile char transmit_USB = 0;
volatile char transmit_USB_complete = 0;
unsigned char start_transmit = 0;

//Tracking variable
unsigned char tracking_pkt_buf[tracking_index];
unsigned char tracking_pkt[tracking_index];
unsigned char UTC[6];
unsigned char LAT[10];
unsigned char LON[11];
unsigned char DATE[10];
unsigned char SPEED[6];
unsigned char COURSE[5];
unsigned char valid_nmea = 0;

unsigned char FLASH_full = 0;
volatile unsigned char new_msg = 0;
unsigned char new_nmea_data;
unsigned char unread_nmea_msg = 0;
volatile unsigned long int adresse; 

//This is the adress for storing in ext. flash, 
//this adress has to be stored in a non-volatile memory
volatile eeprom unsigned long int start_adresse;
   
//Battery
volatile unsigned int volt;
unsigned char charge = 0;
//RTC
unsigned char SECOND;
unsigned char MINUTE;
unsigned char HOUR;
unsigned char DAY;
unsigned char MONTH;
unsigned int YEAR;
char MonthLength[13] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};


void main(void)
{
#asm ("cli");	//Globaly disable interrupts (ISR) during initializing

delay_ms(1000);    //wait for external 32KHz xtal to stabilize

//0=inn 1=ut

//PORTA er ut, kontrollsignaler til display
PORTA=0xFF;
DDRA=0xFF;

// Port B initialization
PORTB=0xFF;
DDRB=0xC7;

// PORTC er ut, datasignaler til display
PORTC=0x00;
DDRC=0xFF;

// Port D initialization
PORTD=0x00;
DDRD=0x78;   //PD0 og PD1 PD4 er utgang

// Port E initialization
PORTE=0xFF;         //enable pull-ups
DDRE=0x00;

// Port F initialization   //for future INS use
PORTF=0x00;
DDRF=0x00;

// Port G initialization   UT
PORTG=0x04;
DDRG=0x7;

// Timer/Counter 0 initialization
// Clock source: TOSC1 pin
// Clock value: TOSC1/128
// Mode: Normal top=FFh
// OC0 output: Disconnected
ASSR=0x08;
TCCR0=0x05;
TCNT0=0x00;
OCR0=0x00;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x01;
ETIMSK=0x00;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: Timer 1 Stopped
// Mode: Normal top=FFFFh
// OC1A output: Discon.
// OC1B output: Discon.
// OC1C output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;
OCR1CH=0x00;
OCR1CL=0x00;

// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: Timer 2 Stopped
// Mode: Normal top=FFh
// OC2 output: Disconnected
TCCR2=0x00;
TCNT2=0x00;
OCR2=0x00;

// Timer/Counter 3 initialization
// Clock source: System Clock
// Clock value: Timer 3 Stopped
// Mode: Normal top=FFFFh
// OC3A output: Discon.
// OC3B output: Discon.
// OC3C output: Discon.
TCCR3A=0x00;
TCCR3B=0x00;
TCNT3H=0x00;
TCNT3L=0x00;
OCR3AH=0x00;
OCR3AL=0x00;
OCR3BH=0x00;
OCR3BL=0x00;
OCR3CH=0x00;
OCR3CL=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
// INT2: Off
// INT3: Off
// INT4: On
// INT5: On
// INT6: On
// INT7: On
EICRA=0x00;
EICRB=0xAA;       //falling edge
EIMSK=0xF0;
EIFR=0xF0;

// USART0 initialization  mot GPSen
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART0 Receiver: On
// USART0 Transmitter: On   //i tilfelle vi m konfigurer uPatch100
// USART0 Mode: Asynchronous
// USART0 Baud rate: 4800
UCSR0A=0x00;
UCSR0B=0x98;
UCSR0C=0x06;
UBRR0H=0x00;
UBRR0L=0x67;

// USART1 initialization  mot USB/PC
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART1 Receiver: On
// USART1 Transmitter: On
// USART1 Mode: Asynchronous
// USART1 Baud rate: 115200
UCSR1A=0x00;
UCSR1B=0x18;
UCSR1C=0x06;
UBRR1H=0x00;
UBRR1L=0x33;             //0x33=9600 og 0x03=115200


// ADC initialization
// ADC Clock frequency: 125,000 kHz
// ADC Voltage Reference: Int., cap. on AREF
// ADC High Speed Mode: Off
ADMUX=0xE7;
ADCSRA=0xEF; 
SFIOR&=0xEF;   


// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
// Analog Comparator Output: Off
ACSR=0x88;
SFIOR=0x00;

// SPI initialization
// SPI Type: Master
// SPI Clock Rate: 2000,000 kHz  x 2
// SPI Clock Phase: Cycle Half
// SPI Clock Polarity: Low
// SPI Data Order: MSB First
SPCR=0x50;
SPSR=0x01; 



//start     

// initial clock and date setting for RTC
SECOND  = 0;
MINUTE  = 0;
HOUR    = 0;
DAY     = 0;
MONTH   = 0;
YEAR    = 0;


init_display();		//Initialize the display

clear_display();	//Make sure there is nothing on the display in the beginning 
		
charge_enable;		//default: Enable charge
TTE_enable;
display_backlight = 1;  //default: display backlight off
DEACTIVATE_FLASH;

//Finds what adress to start at in ext. flash  
//IKKE I BRUK
//find_start_adresse();

flash_delete = 0;
FLASH_full = 0;
unread_nmea_msg = 0;
new_msg = 0;
tracking = 0;	// this is the variable for saving or not saving in FLASH
Write('N', 6, 0); Write('O', 6, 1); Write(' ', 6, 2);
Write('T', 6, 3); Write('R', 6, 4); Write('K', 6, 5);  


#asm ("sei");	

//main loop
while(1)
	{
	//Charges the battery if connected to USB
	//charger();
        
        //Monitores the voltage on the battery
	check_batt();
	
        //Shows clock from the timer0-RTC
        display_clock();
        //Shows date from the timer0-RTC
        display_date();
        
        //Syncronize time and date every minute to GPS(UTC) time.
        if (SECOND == 10)
        	{
        	adjust_time();
        	} 
        
	//Always check for FLASH full
	if (start_adresse > flash_size)  
      		{
      		tracking = 0;
      		FLASH_full = 1;
      		//Write display: FULL
              	Write('F', 5, 10);
              	Write('U', 5, 11);
              	Write('L', 5, 12);
              	Write('L', 5, 13);
      		}
      	
      	//Indicates if storing in FLASH is enabled or not
      	if (tracking == 1)
      		{
      		//Display: TRK
        	Write('T', 6, 0);
        	Write('R', 6, 1);
        	Write('K', 6, 2);
        	Write(' ', 6, 3);
        	Write(' ', 6, 4);
        	Write(' ', 6, 5);
        	}
      	else 
      		{
      	        //Display: NO TRK
               	Write('N', 6, 0);
        	Write('O', 6, 1);
        	Write(' ', 6, 2);
        	Write('T', 6, 3);
        	Write('R', 6, 4);
        	Write('K', 6, 5);
      	        }
        	
      	//Analyze data from the GPS reciever (i.e. NMEA messages)
      	if (unread_nmea_msg == 1)
              	{
              	unread_nmea_msg = 0;
              	
              	//What type of NMEA msg is this?
		nmeaID = nmea_type(nmea_data[2], nmea_data[3], nmea_data[4]);
		
		//Parse nmea msg
		nmea_parse(nmeaID);
        	}
        	
 	//Writes LAT and LON on display
	display_nmea();
        
        //Delete FLASH  	
	if (flash_delete == 1)
		{
		tracking = 0;   //No storing in FLASH while deleting FLASH 
		//Display: NO TRK
                Write('N', 6, 0); Write('O', 6, 1); Write(' ', 6, 2);
                Write('T', 6, 3); Write('R', 6, 4); Write('K', 6, 5);
                //Display: DELETE
                Write('D', 5, 0); Write('E', 5, 1); Write('L', 5, 2);
                Write('E', 5, 3); Write('T', 5, 4); Write('E', 5, 5); 
                bulk_erase();
                adresse = 0;
                start_adresse = 0;
                FLASH_full = 0;
                //Display: Delete letters ("DELETE")
                Write(' ', 5, 0); Write(' ', 5, 1); Write(' ', 5, 2);          
                Write(' ', 5, 3); Write(' ', 5, 4); Write(' ', 5, 5); 
                //Display: Delete letters ("FULL")
                Write(' ', 5, 10); Write(' ', 5, 11);
              	Write(' ', 5, 12); Write(' ', 5, 13);
                flash_delete = 0;
                }
	
	//Transmit to PC via USB
	if (transmit_USB == 1)
		{
		tracking = 0;	//No storing in FLASH while transmiting to PC 
                //Display: NO TRK
                Write('N', 6, 0); Write('O', 6, 1); Write(' ', 6, 2);
                Write('T', 6, 3); Write('R', 6, 4); Write('K', 6, 5); 
                //Display: USB
                Write('U', 5, 0); Write('S', 5, 1); Write('B', 5, 2);
                if(start_transmit == 0)  //This should occour only once per transmit to PC
                	{
                	wait_flash_rdy();
                        ACTIVATE_FLASH;
                        spi(READ);      //Read FLASH from start (adress 0)
                        spi(0x00);      
                        spi(0x00);
                        spi(0x00);
                        UCSR1B = 0x38;	//Enable UDR1E isr
                        start_transmit = 1;
                        }
                if (transmit_USB_complete == 1)
                	{
                	transmit_USB_complete = 0;
                	Write(' ', 5, 0); Write(' ', 5, 1); Write(' ', 5, 2);   //delete spots on display
                	transmit_USB = 0; 
                	start_transmit = 0;
                	}
                }       
		
	}//main loop end	

}//main end



// USART0 Receiver interrupt service routine
// Collects NMEA msg's
#pragma savereg-
interrupt [USART0_RXC] void uart0_rx_isr(void)
{
char status,data; 

#asm
    push r26
    push r27
    push r30
    push r31
    in   r26,sreg
    push r26
#endasm
status=UCSR0A;
data=UDR0;
if ((status & (FRAMING_ERROR | PARITY_ERROR | DATA_OVERRUN))==0)
   	{    
	if ((data == START_NMEA) && (new_msg == 0))
		{
        	new_msg = 1;
        	index = 0;
        	}
	else if ((data != START_NMEA) && (data !=  END_NMEA) && (new_msg == 1))
		{       
		temp_data[index] = data;
		index++;
		}
	else if ((data == END_NMEA) && (new_msg == 1))
       		{       
		//copy temp_data to nmea_data
	     	for (i=0; i < 82; i++)
			{
			nmea_data[i] = temp_data[i];
			}
		unread_nmea_msg = 1;
		index = 0;
		new_msg = 0;
       		}
   	}
#asm
    pop  r26
    out  sreg,r26
    pop  r31
    pop  r30
    pop  r27
    pop  r26
#endasm
}
#pragma savereg+


// USART1 Transmitter interrupt service routine 
// Transmit to PC via USB
#pragma savereg-
interrupt [USART1_DRE] void uart1_dre_isr(void)
{
unsigned char data;
#asm
    push r26
    push r27
    push r30
    push r31
    in   r26,sreg
    push r26
#endasm
data=spi(0x00);
if (data == 0xFF)   	//0xFF means empty FLASH
	{
	UCSR1B = 0x18;	//disable UDR1E isr
	DEACTIVATE_FLASH;
	transmit_USB_complete = 1;
	}                        
else
        {
        UDR1=data;
        }


#asm
    pop  r26
    out  sreg,r26
    pop  r31
    pop  r30
    pop  r27
    pop  r26
#endasm
}
#pragma savereg+

 

// External Interrupt 4 service routine 
// ON/OFF tracking (FLASH)
interrupt [EXT_INT4] void ext_int4_isr(void)
{
if (tracking == 1) 
        {
        tracking = 0;      	//Tracking OFF
        }
else if ((tracking == 0) && (FLASH_full == 0) && (flash_delete == 0))
	{                                            
	tracking = 1;		//Tracking ON
        }
}



// External Interrupt 5 service routine 
// Delete FLASH
interrupt [EXT_INT5] void ext_int5_isr(void)
{
flash_delete = 1;  
}



// External Interrupt 6 service routine 
// Transmit data to PC
interrupt [EXT_INT6] void ext_int6_isr(void)
{
transmit_USB = 1;
}



// External Interrupt 7 service routine
// ON/OFF background light on display
interrupt [EXT_INT7] void ext_int7_isr(void)
{

if(display_backlight == 0)
	{
	display_backlight = 1;         //disable background light
	}
else if (display_backlight == 1);
	{
	display_backlight = 0;		//enable background light	
	}
}
 


// Timer 0 overflow interrupt service routine
// Saves tracking_pkt in FLASH every second and keeps RTC going
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{

//increment the Real Time Clock
RTC();

//Store in FLASH  M25P80     
if ((tracking == 1) && (FLASH_full == 0) && (valid_nmea == 1))
	{
	new_nmea_data = check_new_track();  //new_nmea_data=0 if no new LAT/LON data
	if (new_nmea_data != 0)
        	{
                //Store in FLASH 
                i=0;
                for(adresse=start_adresse; adresse<(start_adresse+tracking_index); adresse++)
                	{
                	PageProgram(tracking_pkt[i], adresse);
                        i++;	
                	}
                start_adresse = start_adresse+tracking_index;
                }
	}


} 




// ADC interrupt service routine 
// Monitoring the battery
interrupt [ADC_INT] void adc_isr(void)
{
unsigned int adc_data;

// Read the AD conversion result
//adc_data=ADCW;	
adc_data=ADCH; 	
adc_data=adc_data+8;	// offset
volt=adc_data*2; 	// Voltage

}

//Displays the voltage of the battery
void check_batt(void)
{
unsigned int tier;
unsigned int hundre;
unsigned int ener;

Write('V',0,7);
//Write('O',0,8);
//Write('L',0,9);
//Write('T',0,10);

if (volt<10)
	{
	Write('0',0,3);
	Write(',',0,4);
	Write('0',0,5);
	Write(volt+48,0,6);
	}
else if ((volt>=10)&&(volt<100))
	{
	Write('0',0,3);
	Write(',',0,4);
	tier=(volt/10);
	Write(tier+48,0,5);
	Write((volt-(tier*10))+48,0,6);
	}
else
	{
	hundre=(volt/100);
	Write(hundre+48,0,3);
	Write(',',0,4);
	tier=(volt-(hundre*100))/10;
	Write(tier+48,0,5);
	ener=volt-((hundre*100)+(tier*10));
	Write(ener+48,0,6);
	} 

        
/*
if (volt=170)
	{
		Write('O',0,8);		// Skriver status ut p display
      		Write('K',0,9);
       		Write(' ',0,10);
	}
else if ((volt<170) && (volt>=165)) 
	{
		Write('L',0,8);
      		Write('O',0,9);
      		Write('W',0,10);
	}           
else
	{
		Write('O',0,8);
		Write('F',0,9);
       		Write('F',0,10);
	}
  */
}


//Converts a value (0-99) to BCD
char CHAR2BCD2(char input)
{
char high = 0;

while (input >= 10)    // Count tens
    	{
	high++;
	input -= 10;
	}

return  (high << 4) | input;        // Add ones and return answer
}



//Just checking the STATx outputs of the BQ24013
//nothing comes out, not even on the oscilloscope.
void charger(void)
{

if(STAT1 == 1)
	{
	Write('1',0,0);
	}
else if (STAT1 == 0)
	{
	Write('0',0,0);
	}
else
	{
	Write(' ',0,0);
	}
	
if(STAT2 == 1)
	{
	Write('1',0,1);
	}
else if(STAT2 == 0)
	{
	Write('0',0,1);
	}
else
	{
	Write(' ',0,1);
	}
}
/*
//Scans the external flashmemory and finds first empty space.
//The adress of this space is used as the first adress in the external flashmemory
//at start-up. 
//BRUKES IKKE
void find_start_adresse(void)
{
unsigned char data = 0;
unsigned char found_adress = 0;

wait_flash_rdy();
ACTIVATE_FLASH;
spi(READ);      //Read FLASH from start (adress 0)
spi(0x00);      
spi(0x00);
spi(0x00);
while (found_adress == 0)
	{
	data=spi(0x00);
        start_adresse++;
        if (data == 0xFF)
        	{
        	found_adress = 1;
        	}
        }
DEACTIVATE_FLASH; 
}
  */