/**************************************************************************
Name:		time.c
Comments:	Functions related to time
**************************************************************************/ 

#include "time.h"

//RTC-module 
//This function is called from th TIMER0 overflow ISR.
//Async. external 32KHz x-tal mode.
void RTC(void)
{
static char LeapMonth;

SECOND++;               // increment second

if (SECOND == 60)
	{
        SECOND = 0;
        MINUTE++;
        
        if (MINUTE > 59)
        	{
            	MINUTE = 0;
            	HOUR++;
            
            	if (HOUR > 23)
                    	{
                        HOUR = 0;
                        DAY++;
        
                        // Check for leap year if month == February
                        if (MONTH == 2)
                            	{
                            	if (!(YEAR & 0x0003))  
                                	{
                                	if (YEAR%100 == 0)
                                    		{
                                    		if (YEAR%400 == 0)
                                        		{LeapMonth = 1;}
        		                        else
                        		                {LeapMonth = 0;}
                                    		}
                                	else
                                    		{LeapMonth = 1;}
                                	}
                            	else
                                	{LeapMonth = 0;}
                            	}
                        else
                            	{LeapMonth = 0;}
        
                        // Check for month length
                        if (DAY > (MonthLength[MONTH] + LeapMonth))
        	                {
                	        DAY = 1;
                        	MONTH++;
        
                            	if (MONTH > 12)
                            		{
                                	MONTH = 1;
                                	YEAR++;
        		                }
                        	}
                    	}
        	}
    	}
}




//Syncronizes the time and date to GPS(UTC) time.
void adjust_time(void)
{
unsigned char HourData[3];
unsigned char MinData[3];
unsigned char SecData[3];

unsigned char YearData[3];
unsigned char MonthData[3];
unsigned char DayData[3];

//uppdate only if we recieved a valid NMEA msg.
if(valid_nmea == 1)
        {
        HourData[0] = UTC[0]; 
        HourData[1] = UTC[1];
        HourData[2] = 0;
        MinData[0] = UTC[2];
        MinData[1] = UTC[3];
        MinData[2] = 0;
        SecData[0] = UTC[4];
        SecData[1] = UTC[5];
        SecData[2] = 0;
        
        DayData[0] = DATE[0];
        DayData[1] = DATE[1];
        DayData[2] = 0;
        MonthData[0] = DATE[2];
        MonthData[1] = DATE[3];
        MonthData[2] = 0;
        YearData[0] = DATE[4];
        YearData[1] = DATE[5];
        YearData[2] = 0;
        
        //ASCII to integer conversion
        SECOND = (unsigned char)atoi(SecData);
        MINUTE = (unsigned char)atoi(MinData);
        HOUR = (unsigned char)atoi(HourData);
        
        DAY = (unsigned char)atoi(DayData);
        MONTH = (unsigned char)atoi(MonthData);
        YEAR = (unsigned char)atoi(YearData);
        }
}



//Displays the clock (UTC)  
void display_clock(void)
{
unsigned char HH, HL, MH, ML, SH, SL;

HH = CHAR2BCD2(HOUR);
HL = (HH & 0x0F) + '0';
HH = (HH >> 4) + '0';

MH = CHAR2BCD2(MINUTE);
ML = (MH & 0x0F) + '0';
MH = (MH >> 4) + '0';

SH = CHAR2BCD2(SECOND);
SL = (SH & 0x0F) + '0';
SH = (SH >> 4) + '0';

//Display: UTC <clock>
Write('U', 3, 0); Write('T', 3, 1); Write('C', 3, 2); Write(' ', 3, 3); Write(' ', 3, 4); 
Write(HH, 3, 5);
Write(HL, 3, 6);
Write(':', 3, 7);
Write(MH, 3, 8);
Write(ML, 3, 9);
Write(':', 3, 10);
Write(SH, 3, 11);
Write(SL, 3, 12);
}



//Displays the date
void display_date(void)
{
char YH, YL, MH, ML, DH, DL;

YH = CHAR2BCD2(YEAR);
YL = (YH & 0x0F) + '0';
YH = (YH >> 4) + '0';

MH = CHAR2BCD2(MONTH);
ML = (MH & 0x0F) + '0';
MH = (MH >> 4) + '0';

DH = CHAR2BCD2(DAY);
DL = (DH & 0x0F) + '0';
DH = (DH >> 4) + '0'; 

//Display: DTE <date>
Write('D', 4, 0); Write('T', 4, 1); Write('E', 4, 2); Write(' ', 4, 3);  Write(' ', 4, 4);
Write(DH, 4, 5);
Write(DL, 4, 6);
Write('.', 4, 7);
Write(MH, 4, 8);
Write(ML, 4, 9);
Write('.', 4, 10);
Write(YH, 4, 11);
Write(YL, 4, 12); 
}

